@@ -27,7 +27,11 @@ class KAboutData;
 class KPluginMetaData;
 namespace KCrash
 {
+#ifdef KCOREADDONS_STATIC
+void defaultCrashHandler(int sig);
+#else
 Q_DECL_IMPORT void defaultCrashHandler(int sig);
+#endif
 }
 
 /**
@@ -1149,8 +1153,9 @@ public:
      * @return the program's icon name.
      *
      * This is mostly for compatibility, given that setProgramIconName is deprecated.
+     * @deprecated Since 5.2, Use QGuiApplication::windowIcon instead
      */
-    KCOREADDONS_DEPRECATED_VERSION(5, 2, "Use QApplication::windowIcon")
+    KCOREADDONS_DEPRECATED_VERSION(5, 2, "Use QGuiApplication::windowIcon instead")
     QString programIconName() const;
 #endif
 
