@@ -5,7 +5,7 @@
     SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
     SPDX-FileCopyrightText: 2006 Kevin Ottens <ervin@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.0-only
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KJOB_H
@@ -88,11 +88,13 @@ public:
      * @see totalAmount
      */
     enum Unit {
-        Bytes, ///< Directory and file sizes in bytes
+        Bytes = 0, ///< Directory and file sizes in bytes
         Files, ///< The number of files handled by the job
         Directories, ///< The number of directories handled by the job
         Items, ///< The number of items (e.g. both directories and files) handled by the job
                ///< @since 5.72
+
+        UnitsCount, ///< @internal since 5.87, used internally only, do not use.
     };
     Q_ENUM(Unit)
 
@@ -370,7 +372,7 @@ Q_SIGNALS:
      * Emitted when the job is finished, in any case. It is used to notify
      * observers that the job is terminated and that progress can be hidden.
      *
-     * @since 5.75: this signal is guaranteed to be emitted exactly once.
+     * Since 5.75 this signal is guaranteed to be emitted exactly once.
      *
      * This is a private signal, it can't be emitted directly by subclasses of
      * KJob, use emitResult() instead.
@@ -425,7 +427,7 @@ Q_SIGNALS:
     /**
      * Emitted when the job is finished (except when killed with KJob::Quietly).
      *
-     * @since 5.75: this signal is guaranteed to be emitted at most once.
+     * Since 5.75 this signal is guaranteed to be emitted at most once.
      *
      * Use error to know if the job was finished with error.
      *
