@@ -1,19 +1,10 @@
-/*  This file is part of the KDE project
-    Copyright (C) 2014 Alex Richardson <arichardson.kde@gmail.com>
+/*
+    This file is part of the KDE project
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2 as published by the Free Software Foundation.
+    SPDX-FileCopyrightText: 2014 Alex Richardson <arichardson.kde@gmail.com>
+    SPDX-FileCopyrightText: 2021 Alexander Lohnau <alexander.lohnau@gmx.de>
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KPLUGINMETADATA_H
@@ -23,20 +14,19 @@
 
 #include <QExplicitlySharedDataPointer>
 #include <QJsonObject>
+#include <QMetaType>
 #include <QString>
 #include <QStringList>
-#include <QMetaType>
 
 #include <functional>
 
 class KPluginLoader;
+class KPluginFactory;
 class QPluginLoader;
-class QStringList;
 class KPluginMetaDataPrivate;
 class KAboutPerson;
 class QObject;
-
-
+class QStaticPlugin;
 /**
  * @class KPluginMetaData kpluginmetadata.h KPluginMetaData
  *
@@ -79,15 +69,13 @@ class QObject;
         "Name": "Date and Time",
         "Description": "Date and time by timezone",
         "Icon": "preferences-system-time",
-        "Authors": { "Name": "Aaron Seigo", "Email": "aseigo@kde.org" },
+        "Authors": [ { "Name": "Aaron Seigo", "Email": "aseigo@kde.org" } ],
         "Category": "Date and Time",
-        "Dependencies": [],
         "EnabledByDefault": "true",
         "License": "LGPL",
         "Id": "time",
         "Version": "1.0",
-        "Website": "https://plasma.kde.org/",
-        "ServiceTypes": ["Plasma/DataEngine"]
+        "Website": "https://plasma.kde.org/"
      }
    }
    @endverbatim
@@ -97,16 +85,51 @@ class QObject;
  */
 class KCOREADDONS_EXPORT KPluginMetaData
 {
-public:
+    Q_GADGET
+    Q_PROPERTY(bool isValid READ isValid CONSTANT)
+    Q_PROPERTY(bool isHidden READ isHidden CONSTANT)
+    Q_PROPERTY(QString fileName READ fileName CONSTANT)
+    Q_PROPERTY(QString metaDataFileName READ metaDataFileName CONSTANT)
+    Q_PROPERTY(QJsonObject rawData READ rawData CONSTANT)
+    Q_PROPERTY(QString name READ name CONSTANT)
+    Q_PROPERTY(QString description READ description CONSTANT)
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 87)
+    Q_PROPERTY(QString extraInformation READ extraInformation CONSTANT)
+#endif
+    Q_PROPERTY(QVariantList authors READ authorsVariant CONSTANT)
+    Q_PROPERTY(QVariantList translators READ translatorsVariant CONSTANT)
+    Q_PROPERTY(QVariantList otherContributors READ otherContributorsVariant CONSTANT)
+    Q_PROPERTY(QString category READ category CONSTANT)
+    Q_PROPERTY(QString iconName READ iconName CONSTANT)
+    Q_PROPERTY(QString license READ license CONSTANT)
+    Q_PROPERTY(QString licenseText READ licenseText CONSTANT)
+    Q_PROPERTY(QString copyrightText READ copyrightText CONSTANT)
+    Q_PROPERTY(QString pluginId READ pluginId CONSTANT)
+    Q_PROPERTY(QString version READ version CONSTANT)
+    Q_PROPERTY(QString website READ website CONSTANT)
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 79)
+    Q_PROPERTY(QStringList dependencies READ dependencies CONSTANT)
+#endif
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 89)
+    Q_PROPERTY(QStringList serviceTypes READ serviceTypes CONSTANT)
+#endif
+    Q_PROPERTY(QStringList mimeTypes READ mimeTypes CONSTANT)
+    Q_PROPERTY(QStringList formFactors READ formFactors CONSTANT)
+    Q_PROPERTY(bool isEnabledByDefault READ isEnabledByDefault CONSTANT)
+    Q_PROPERTY(int initialPreference READ isEnabledByDefault CONSTANT)
 
+public:
     /** Creates an invalid KPluginMetaData instance */
     KPluginMetaData();
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 86)
     /**
      * Reads the plugin metadata from a KPluginLoader instance. You must call KPluginLoader::setFileName()
      * or use the appropriate constructor on @p loader before calling this.
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 86, "Use KPluginMetaData(QPluginLoader) instead")
     KPluginMetaData(const KPluginLoader &loader);
+#endif
 
     /**
      * Reads the plugin metadata from a QPluginLoader instance. You must call QPluginLoader::setFileName()
@@ -153,6 +176,17 @@ public:
      */
     KPluginMetaData(const QJsonObject &metaData, const QString &pluginFile, const QString &metaDataFile);
 
+    /*
+     * Constructs a KPluginMetaData from the static plugin.
+     * If it does not have any meta data the @p metaData value is used
+     *
+     * @see KPluginFactory::loadFactory
+     * @see KPluginFactory::instantiatePlugin
+     *
+     * @since 5.89
+     */
+    KPluginMetaData(QStaticPlugin plugin, const QJsonObject &metaData = {});
+
     /**
      * Copy contructor
      */
@@ -167,7 +201,7 @@ public:
     ~KPluginMetaData();
 
     /**
-     * Load a KPluginMetaData instace from a .desktop file. Unlike the constructor which takes
+     * Load a KPluginMetaData instance from a .desktop file. Unlike the constructor which takes
      * a single file parameter this method allows you to specify which service type files should
      * be parsed to determine the correct type for a given .desktop property.
      * This ensures that a e.g. comma-separated string list field in the .desktop file will correctly
@@ -186,6 +220,41 @@ public:
      */
     static KPluginMetaData fromDesktopFile(const QString &file, const QStringList &serviceTypes = QStringList());
 
+    /**
+     * Load a KPluginMetaData instance from a .json file. Unlike the constructor with a single file argument,
+     * this ensure that only JSON format plugins are loaded and any other type is rejected.
+     *
+     * @param jsonFile the .json file to load
+     * @since 5.91
+     */
+    static KPluginMetaData fromJsonFile(const QString &jsonFile);
+
+    /**
+     * @param directory The directory to search for plugins. If a relative path is given for @p directory,
+     * all entries of QCoreApplication::libraryPaths() will be checked with @p directory appended as a
+     * subdirectory. If an absolute path is given only that directory will be searched.
+     * @note Check if the returned KPluginMetaData is valid before continuing to use it.
+     *
+     * @param pluginId The Id of the plugin. The id should be the same as the filename, see KPluginMetaData::pluginId()
+     * @since 5.84
+     */
+    static KPluginMetaData findPluginById(const QString &directory, const QString &pluginId);
+
+    /**
+     * Find all plugins inside @p directory. Only plugins which have JSON metadata will be considered.
+     *
+     * @param directory The directory to search for plugins. If a relative path is given for @p directory,
+     * all entries of QCoreApplication::libraryPaths() will be checked with @p directory appended as a
+     * subdirectory. If an absolute path is given only that directory will be searched.
+     *
+     * @param filter a callback function that returns @c true if the found plugin should be loaded
+     * and @c false if it should be skipped. If this argument is omitted all plugins will be loaded
+     *
+     * @return all plugins found in @p directory that fulfil the constraints of @p filter
+     * @since 5.86
+     */
+    static QVector<KPluginMetaData> findPlugins(const QString &directory, std::function<bool(const KPluginMetaData &)> filter = {});
+
     /**
      * @return whether this object holds valid information about a plugin.
      * If this is @c true pluginId() will return a non-empty string.
@@ -223,16 +292,20 @@ public:
      */
     QJsonObject rawData() const;
 
-
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 86)
     /**
      * Tries to instantiate this plugin using KPluginMetaData::fileName().
      * @note The value of KPluginMetaData::dependencies() is not used here, dependencies must be
      * resolved manually.
      *
      * @return The plugin root object or @c nullptr if it could not be loaded
-     * @see QPluginLoader::instance(), KPluginLoader::instance()
+     * @see QPluginLoader::instance(), KPluginFactory::loadFactory, KPluginFactory::instantiatePlugin
+     * @deprecated Since 5.86, use @ref KPluginFactory::loadFactory or @ref KPluginFactory::instantiatePlugin when using
+     * KPluginFactory. Otherwise use QPluginLoader::instance() instead.
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 86, "See API docs")
     QObject *instantiate() const;
+#endif
 
     /**
      * @return the user visible name of the plugin.
@@ -244,12 +317,16 @@ public:
      */
     QString description() const;
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 87)
     /**
      * @return additional information about this plugin (e.g. for use in an "about plugin" dialog)
      *
      * @since 5.18
+     * @deprecated Since 5.87, deprecate for lack of usage. Use a meaningful custom key in the json metadata instead
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 87, "Deprecate for lack of usage, use a meaningful custom key in the json metadata instead")
     QString extraInformation() const;
+#endif
 
     /**
      * @return the author(s) of this plugin.
@@ -287,6 +364,12 @@ public:
      */
     QString license() const;
 
+    /**
+     * @return the text of the license, equivalent to KAboutLicense::byKeyword(license()).text()
+     * @since 5.73
+     */
+    QString licenseText() const;
+
     /**
      * @return a short copyright statement
      *
@@ -295,9 +378,9 @@ public:
     QString copyrightText() const;
 
     /**
-     * @return the internal name of the plugin (for KParts Plugins this is
-     * the same name as set in the .rc file). If the plugin name property is not set in
-     * the metadata this will return the plugin file name without the file extension.
+     * @return the internal name of the plugin
+     * If the Id property is not set in the metadata, this will return the
+     * plugin file name without the file extension.
      */
     QString pluginId() const;
 
@@ -311,12 +394,17 @@ public:
      */
     QString website() const;
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 79)
     /**
      * @return a list of plugins that this plugin depends on so that it can function properly
      * @see KJsonPluginInfo::pluginId()
+     * @deprecated Since 5.79, plugin dependencies are deprecated and will be removed in KF6
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 79, "Plugin dependencies are deprecated and will be removed in KF6")
     QStringList dependencies() const;
+#endif
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 89)
     /**
      * Returns the service types that this plugin implements.
      *
@@ -328,8 +416,11 @@ public:
      * @note Unlike KService this does not contain the MIME types. To get the handled MIME types
      * use the KPluginMetaData::mimeTypes() function.
      * @return a list of service types this plugin implements (e.g. "Plasma/DataEngine")
+     * @dprecated Since 5.89, use dedicated plugin namespaces instead to filter plugins of a specific type
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 89, "See API docs")
     QStringList serviceTypes() const;
+#endif
 
     /**
      * @return a list of MIME types this plugin can handle (e.g. "application/pdf", "image/png", etc.)
@@ -369,6 +460,22 @@ public:
      */
     int initialPreference() const;
 
+    /**
+     * Returns @c true if the plugin is enabled in @p config, otherwise returns isEnabledByDefault().
+     * This can be used in conjunctionwith KPluginWidget/KPluginSelector.
+     *
+     * The @p config param should be a KConfigGroup object, because KCoreAddons can not depend
+     * on KConfig directly, this parameter is a template.
+     * @param config KConfigGroup where the enabled state is stored
+     * @since 5.89
+     */
+    template<typename T>
+    bool isEnabled(const T &config) const
+    {
+        Q_ASSERT(config.isValid());
+        return config.readEntry(pluginId() + QLatin1String("Enabled"), isEnabledByDefault());
+    }
+
     /**
      * @return the value for @p key from the metadata or @p defaultValue if the key does not exist
      * or the value for @p key is not of type string
@@ -377,27 +484,74 @@ public:
      */
     QString value(const QString &key, const QString &defaultValue = QString()) const;
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 88)
+    /**
+     * Overload to make sure the bool overload is not taken by accident
+     * @overload
+     * @since 5.88
+     */
+    KCOREADDONS_DEPRECATED_VERSION(5, 88, "Construct a QString instead of using a char array, otherwise there the bool overload could be chosen by accident")
+    QString value(const QString &key, const char *ch) const
+    {
+        return value(key, QString::fromLatin1(ch));
+    }
+#else
+    QString value(const QString &key, const char *ch) const = delete;
+#endif
+
+    /**
+     * @overload
+     * @since 5.88
+     */
+    bool value(const QString &key, bool defaultValue) const;
+
+    /**
+     * @overload
+     * @since 5.88
+     */
+    int value(const QString &key, int defaultValue) const;
+
     /** @return the value for @p key inside @p jo as a string list. If the type of @p key is string, a list with containing
      * just that string will be returned, if it is an array the list will contain one entry for each array member.
      * If the key cannot be found an empty list will be returned.
+     * @overload
+     * @since 5.88
      */
-    static QStringList readStringList(const QJsonObject &jo, const QString &key);
+    QStringList value(const QString &key, const QStringList &defaultValue) const;
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 88)
+    /** @return the value for @p key inside @p jo as a string list. If the type of @p key is string, a list with containing
+     * just that string will be returned, if it is an array the list will contain one entry for each array member.
+     * If the key cannot be found an empty list will be returned.
+     * @deprecated Since 5.88, use @p value(QString, QStringList) on KPluginMetaData instance instead
+     */
+    KCOREADDONS_DEPRECATED_VERSION(5, 88, "Use value(QString, QStringList) on KPluginMetaData instance instead")
+    static QStringList readStringList(const QJsonObject &jo, const QString &key);
+#endif
+
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 88)
     /**
      * Reads a value from @p jo but unlike QJsonObject::value() it allows different entries for each locale
      * This is done by appending the locale identifier in brackets to the key (e.g. "[de_DE]" or "[es]")
      * When looking for a key "foo" with German (Germany) locale we will first attempt to read "foo[de_DE]",
      * if that does not exist "foo[de]", finally falling back to "foo" if that also doesn't exist.
      * @return the translated value for @p key from @p jo or @p defaultValue if @p key was not found
+     * @deprecated Since 5.88, use KJsonUtils::readTranslatedValue instead
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 88, "Use KJsonUtils::readTranslatedValue instead")
     static QJsonValue readTranslatedValue(const QJsonObject &jo, const QString &key, const QJsonValue &defaultValue = QJsonValue());
+#endif
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 88)
     /**
      * @return the translated value of @p key from @p jo as a string or @p defaultValue if @p key was not found
      * or the value for @p key is not of type string
      * @see KPluginMetaData::readTranslatedValue(const QJsonObject &jo, const QString &key)
+     * @deprecated Since 5.88, use KJsonUtils::readTranslatedString instead
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 88, "Use KJsonUtils::readTranslatedString instead")
     static QString readTranslatedString(const QJsonObject &jo, const QString &key, const QString &defaultValue = QString());
+#endif
 
     /**
      * @return @c true if this object is equal to @p other, otherwise @c false
@@ -411,16 +565,38 @@ public:
     {
         return !(*this == other);
     }
+
+    /**
+     * @note for loading plugin the plugin independently of it being static/dynamic
+     * use @ref KPluginFactory::loadFactory or @ref KPluginFactory::instantiatePlugin.
+     * @return true if the instance represents a static plugin
+     * @since 5.89
+     */
+    bool isStaticPlugin() const;
+
 private:
     QJsonObject rootObject() const;
     void loadFromDesktopFile(const QString &file, const QStringList &serviceTypes);
+    void loadFromJsonFile(const QString &file);
+
 private:
+    QVariantList authorsVariant() const;
+    QVariantList translatorsVariant() const;
+    QVariantList otherContributorsVariant() const;
+    QStaticPlugin staticPlugin() const;
+    QString requestedFileName() const;
+
     QJsonObject m_metaData;
     QString m_fileName;
     QExplicitlySharedDataPointer<KPluginMetaDataPrivate> d; // for future binary compatible extensions
+    friend class KPluginFactory;
 };
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+inline size_t qHash(const KPluginMetaData &md, size_t seed)
+#else
 inline uint qHash(const KPluginMetaData &md, uint seed)
+#endif
 {
     return qHash(md.pluginId(), seed);
 }
